@@ -1,5 +1,40 @@
+/* dvi.c */
+
+/************************************************************************
+
+  Part of the dvipng distribution
+
+  This program is free software; you can redistribute it and/or modify
+  it under the terms of the GNU General Public License as published by
+  the Free Software Foundation; either version 2 of the License, or
+  (at your option) any later version.
+
+  This program is distributed in the hope that it will be useful, but
+  WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+  General Public License for more details.
+
+  You should have received a copy of the GNU General Public License
+  along with this program; if not, write to the Free Software
+  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
+  02111-1307, USA.
+
+  Copyright  2002-2004 Jan-ke Larsson
+
+************************************************************************/
+
 #include "dvipng.h"
-#include <libgen.h>
+#ifdef MIKTEX
+# include <gnu-miktex.h>
+# define SLEEP    Sleep(1000)
+#else  /* MIKTEX */
+# ifdef HAVE_LIBGEN_H
+#  include <libgen.h>
+# else
+# define basename xbasename
+#endif
+# define SLEEP    sleep(1)
+#endif	/* MIKTEX */
 #include <sys/stat.h>
 
 bool followmode=0;
@@ -14,7 +49,7 @@ static unsigned char fgetc_follow(FILE* fp)
   int got=fgetc(fp);
 
   while(followmode && got==EOF) {
-    sleep(1);
+    SLEEP;
     got=fgetc(fp);
   }
   if (got==EOF)
@@ -58,6 +93,7 @@ void DVIInit(struct dvi_data* dvi)
   Message(BE_VERBOSE,"'%.*s' -> %s\n",k,pre+15,dvi->outname);
   fstat(fileno(dvi->filep), &stat);
   dvi->mtime = stat.st_mtime;
+  dvi->pagelistp=NULL;
 }
 
 struct dvi_data* DVIOpen(char* dviname,char* outname)
@@ -65,7 +101,7 @@ struct dvi_data* DVIOpen(char* dviname,char* outname)
   char* tmpstring;
   struct dvi_data* dvi;
 
-  if ((dvi = malloc(sizeof(struct dvi_data)))==NULL)
+  if ((dvi = calloc(1,sizeof(struct dvi_data)))==NULL)
     Fatal("cannot allocate memory for dvi struct");
 
   dvi->type = DVI_TYPE;
@@ -81,8 +117,11 @@ struct dvi_data* DVIOpen(char* dviname,char* outname)
   
   if (outname==NULL) { 
     dvi->outname = malloc(strlen(basename(dviname))+7);
-    if (dvi->outname==NULL)
+    if (dvi->outname==NULL) {
+      free(dvi->name);
+      free(dvi);
       Fatal("cannot allocate space for output filename");
+    }
     strcpy(dvi->outname,basename(dviname));
     tmpstring = strrchr(dvi->outname, '.');
     if (tmpstring != NULL) 
@@ -90,8 +129,11 @@ struct dvi_data* DVIOpen(char* dviname,char* outname)
     strcat(dvi->outname, "%d.png");
   } else {
     dvi->outname = malloc(strlen(outname)+1);
-    if (dvi->outname==NULL)
+    if (dvi->outname==NULL) {
+      free(dvi->name);
+      free(dvi);
       Fatal("cannot allocate space for output filename");
+    }
     strcpy(dvi->outname,outname);
   }
 
@@ -102,7 +144,7 @@ struct dvi_data* DVIOpen(char* dviname,char* outname)
     dvi->filep = fopen(dvi->name,"rb");
   }
   while((dvi->filep == NULL) && followmode) {
-    sleep(1);
+    SLEEP;
     *tmpstring='.';
     if ((dvi->filep = fopen(dvi->name,"rb")) == NULL) {
       /* do not insist on .dvi */
@@ -111,6 +153,9 @@ struct dvi_data* DVIOpen(char* dviname,char* outname)
     }
   }
   if (dvi->filep == NULL) {
+    free(dvi->name);
+    free(dvi->outname);
+    free(dvi);
     perror(dviname);
     exit (EXIT_FAILURE);
   }
@@ -206,6 +251,13 @@ void SkipPage(struct dvi_data* dvi)
       DEBUG_PRINT(DEBUG_DVI,("NOSKIP CMD:\t%s", dvi_commands[*command]));
       FontDef(command,dvi);
       break;
+    case XXX1: case XXX2: case XXX3: case XXX4:
+      DEBUG_PRINT(DEBUG_DVI,("NOSKIP CMD:\t%s %d", dvi_commands[*command],
+			     UNumRead(command+1, dvi_commandlength[*command]-1)));
+      SetSpecial(command + dvi_commandlength[*command], 
+		 UNumRead(command+1, dvi_commandlength[*command]-1),
+		 0,0);
+      break;
     case BOP: case PRE: case POST: case POST_POST:
       Fatal("%s occurs within page", dvi_commands[*command]);
       break;
@@ -241,18 +293,20 @@ struct page_list* InitPage(struct dvi_data* dvi)
     }
     command=DVIGetCommand(dvi);
   }
-  if ((tpagelistp = malloc(sizeof(struct page_list)))==NULL)
+  if ((tpagelistp = 
+       malloc(sizeof(struct page_list)+(csp+1-2)*sizeof(struct dvi_color)))==NULL)
     Fatal("cannot allocate memory for new page entry");
   tpagelistp->next = NULL;
   if ( *command == BOP ) {  /*  Init page */
     int i;
     DEBUG_PRINT(DEBUG_DVI,("PAGE START:\tBOP"));
+    StoreColorStack(tpagelistp);
     tpagelistp->offset = ftell(dvi->filep)-45;
     for (i = 0; i <= 9; i++) {
       tpagelistp->count[i] = UNumRead(command + 1 + i*4, 4);
       DEBUG_PRINT(DEBUG_DVI,(" %d",tpagelistp->count[i]));
     }
-    if (dvi->pagelistp==NULL)
+    if (dvi->pagelistp==NULL) 
       tpagelistp->count[10] = 1;
     else
       tpagelistp->count[10] = dvi->pagelistp->count[10]+1;
@@ -261,12 +315,14 @@ struct page_list* InitPage(struct dvi_data* dvi)
     DEBUG_PRINT(DEBUG_DVI,("DVI END:\tPOST"));
     tpagelistp->offset = ftell(dvi->filep)-1;
     tpagelistp->count[0] = PAGE_POST; /* POST */
+    tpagelistp->count[10] = PAGE_POST; /* POST */
   }
   return(tpagelistp);
 }
 
 int SeekPage(struct dvi_data* dvi, struct page_list* page)
 {
+  ReadColorStack(page);
   return(fseek(dvi->filep, 
 	       page->offset+((page->count[0]==PAGE_POST) ? 1L : 45L),
 	       SEEK_SET));
@@ -351,11 +407,13 @@ void DelPageList(struct dvi_data* dvi)
 
 void DVIClose(struct dvi_data* dvi)
 {
-  fclose(dvi->filep);
-  DelPageList(dvi);
-  free(dvi->outname);
-  free(dvi->name);
-  free(dvi);
+  if (dvi!=NULL) {
+    fclose(dvi->filep);
+    DelPageList(dvi);
+    free(dvi->outname);
+    free(dvi->name);
+    free(dvi);
+  }
 }
 
 bool DVIReOpen(struct dvi_data* dvi)
@@ -367,7 +425,7 @@ bool DVIReOpen(struct dvi_data* dvi)
     dvi->filep=NULL;
     DelPageList(dvi);
     while(((dvi->filep = fopen(dvi->name,"rb")) == NULL) && followmode) {
-      sleep(1);
+      SLEEP;
     }
     if (dvi->filep == NULL) {
       perror(dvi->name);
