@@ -1,11 +1,32 @@
-#include "dvipng.h"
-#include <fcntl.h> // open/close
-#include <sys/mman.h>
-#include <sys/stat.h>
+/* fontmap.c */
 
-static unsigned char* psfont_mmap = (unsigned char *)-1;
-static int psfont_filedes = -1;
-static struct stat psfont_stat;
+/************************************************************************
+
+  Part of the dvipng distribution
+
+  This program is free software; you can redistribute it and/or modify
+  it under the terms of the GNU General Public License as published by
+  the Free Software Foundation; either version 2 of the License, or
+  (at your option) any later version.
+
+  This program is distributed in the hope that it will be useful, but
+  WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+  General Public License for more details.
+
+  You should have received a copy of the GNU General Public License
+  along with this program; if not, write to the Free Software
+  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
+  02111-1307, USA.
+
+  Copyright  2002-2004 Jan-ke Larsson
+
+************************************************************************/
+
+#include "dvipng.h"
+
+static char* psfont_name=NULL;
+static struct filemmap psfont_mmap;
 static struct psfontmap *psfontmap=NULL;
 
 inline char* newword(char** buffer, char* end) 
@@ -26,7 +47,7 @@ char* find_format(char* name)
 {
   /* Cater for both new (first case) and old (second case) kpathsea */
   char* format =
-     kpse_find_file(name,kpse_fontmap_format,false);
+    kpse_find_file(name,kpse_fontmap_format,false);
 
   if (format==NULL)
     format = kpse_find_file(name,kpse_dvips_config_format,false);
@@ -35,61 +56,75 @@ char* find_format(char* name)
 
 void InitPSFontMap(void)
 {
-  char *pos,*end,*psfont_name = find_format("ps2pk.map");
+  char *pos,*end;
   struct psfontmap *entry;
 
+  psfont_name=find_format("ps2pk.map");
   if (psfont_name==NULL)
-    psfont_name = find_format("psfonts.map");
+    psfont_name=find_format("psfonts.map");
 
   if (psfont_name==NULL) {
     Warning("cannot find ps2pk.map, nor psfonts.map");
     return;
   }
   DEBUG_PRINT(DEBUG_FT,("\n  OPEN PSFONT MAP:\t'%s'", psfont_name));  
-  if ((psfont_filedes = open(psfont_name,O_RDONLY)) == -1) { 
+  if (MmapFile(psfont_name,&psfont_mmap)) {
     Warning("psfonts map %s could not be opened", psfont_name);
     return;
   }
-  fstat(psfont_filedes,&psfont_stat);
-  psfont_mmap = mmap(NULL,psfont_stat.st_size,
-		     PROT_READ, MAP_SHARED,psfont_filedes,0);
-  if (psfont_mmap == (unsigned char *)-1) 
-    Warning("cannot mmap psfonts map %s !\n",psfont_name);
-  else {
-    pos = psfont_mmap;
-    end = psfont_mmap+psfont_stat.st_size;
-    while(pos<end) {
-      while(pos < end && (*pos=='\n' || *pos==' ' || *pos=='\t' 
-			  || *pos=='%' || *pos=='*' || *pos==';' || *pos=='#')) {
-	while(pos < end && *pos!='\n') pos++; /* skip comments/empty rows */
-	pos++;
-      }
-      if (pos < end) {
-	if ((entry=malloc(sizeof(struct psfontmap)))==NULL)
-	  Fatal("cannot malloc psfontmap space");
-	entry->line = pos;
-	/* skip <something and quoted entries */
-	while(pos < end && (*pos=='<' || *pos=='"')) {
-	  if (*pos=='<') 
-	    while(pos < end && *pos!=' ' && *pos!='\t') pos++;
-	  else 
-	    while(pos < end && *pos!='"') pos++;
-	  while(pos < end && (*pos==' ' || *pos=='\t')) pos++;
-	}
-	/* first word is font name */
- 	entry->tfmname = newword(&pos,end);
-	entry->psfile = NULL;
-	entry->encname = NULL;
-	while(pos < end && *pos!='\n') pos++;
-	entry->end = pos;
-	entry->next=psfontmap;
-	psfontmap=entry;
-      }
+  pos = psfont_mmap.mmap;
+  end = psfont_mmap.mmap+psfont_mmap.size;
+  while(pos<end) {
+    while(pos < end && (*pos=='\n' || *pos==' ' || *pos=='\t' 
+			|| *pos=='%' || *pos=='*' || *pos==';' || *pos=='#')) {
+      while(pos < end && *pos!='\n') pos++; /* skip comments/empty rows */
       pos++;
     }
+    if (pos < end) {
+      if ((entry=malloc(sizeof(struct psfontmap)))==NULL)
+	Fatal("cannot malloc psfontmap space");
+      entry->line = pos;
+      /* skip <something and quoted entries */
+      while(pos < end && (*pos=='<' || *pos=='"')) {
+	if (*pos=='<') 
+	  while(pos < end && *pos!=' ' && *pos!='\t') pos++;
+	else 
+	  while(pos < end && *pos!='"') pos++;
+	while(pos < end && (*pos==' ' || *pos=='\t')) pos++;
+      }
+      /* first word is font name */
+      entry->tfmname = newword(&pos,end);
+      entry->psfile = NULL;
+      entry->encname = NULL;
+      entry->encoding = NULL;
+      while(pos < end && *pos!='\n') pos++;
+      entry->end = pos;
+      entry->next=psfontmap;
+      psfontmap=entry;
+    }
+    pos++;
   }
 }
 
+void ClearPSFontMap(void)
+{
+  struct psfontmap *entry;
+
+  while(psfontmap!=NULL) {
+    entry=psfontmap;
+    psfontmap=psfontmap->next;
+    free(entry->tfmname);
+    if (entry->psfile!=NULL)
+      free(entry->psfile);
+    if (entry->encname!=NULL)
+      free(entry->encname);
+    free(entry);
+  }
+  UnMmapFile(&psfont_mmap);
+  if (psfont_name != NULL)
+    free(psfont_name);
+  psfont_name=NULL;
+}
 
 struct psfontmap* FindPSFontMap(char* fontname)
 {
@@ -212,3 +247,4 @@ struct psfontmap* FindPSFontMap(char* fontname)
   
   return(entry);
 }
+
