@@ -1,5 +1,37 @@
+/* misc.c */
+
+/************************************************************************
+
+  Part of the dvipng distribution
+
+  This program is free software; you can redistribute it and/or modify
+  it under the terms of the GNU General Public License as published by
+  the Free Software Foundation; either version 2 of the License, or
+  (at your option) any later version.
+
+  This program is distributed in the hope that it will be useful, but
+  WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+  General Public License for more details.
+
+  You should have received a copy of the GNU General Public License
+  along with this program; if not, write to the Free Software
+  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
+  02111-1307, USA.
+
+  Copyright  2002-2004 Jan-ke Larsson
+
+************************************************************************/
+
 #include "dvipng.h"
-#include <libgen.h>
+#ifdef HAVE_LIBGEN_H
+# include <libgen.h>
+#else
+#define basename xbasename
+#endif
+#include <fcntl.h> // open/close
+#include <sys/mman.h>
+#include <sys/stat.h>
 
 static char *programname;
 
@@ -10,10 +42,19 @@ static char *programname;
 bool DecodeArgs(int argc, char ** argv)
 {
   int     i;                 /* argument index for options     */
-  bool ppused=_FALSE;        /* Flag when -pp is used          */
+  bool ppused=false;        /* Flag when -pp is used          */
   char *dviname=NULL;        /* Name of dvi file               */
   char *outname=NULL;        /* Name of output file            */
 
+#ifdef HAVE_GDIMAGEGIF
+  programname=strrchr(argv[0],'/');
+  if (programname!=NULL)
+    programname++;
+  else
+    programname=argv[0];
+  if (strncmp(programname,"dvigif",6)==0)
+    flags |= GIF_OUTPUT;
+#endif
   programname=argv[0];
 
   for (i=1; i<argc; i++) {
@@ -25,22 +66,10 @@ bool DecodeArgs(int argc, char ** argv)
 	c=argv[i][2];
       }
       switch (c) {
-      case 'n':       /* no-image-on-warn */
-	if (strncmp(p,"o-image-on-warn",18)==0) { 
-	  if (p[9] != '0') {
-	    flags |= NO_IMAGE_ON_WARN;
-	    Message(PARSE_STDIN,"No images output for pages with warnings\n",p);
-	  } else {
-	    flags &= ~NO_IMAGE_ON_WARN;
-	    Message(PARSE_STDIN,"Images output even for pages with warnings\n");
-	  }
-	  break;
-	}
-	goto DEFAULT;
       case 'd':       /* selects Debug output */
 	if (*p>'9' && *p!='-') {
-	  if (strncmp(p,"vinum",8)==0) { 
-	    if (p[9] != '0') {
+	  if (strncmp(p,"vinum",5)==0) { 
+	    if (p[5] != '0') {
 	      flags |= DVI_PAGENUM;
 	      Message(PARSE_STDIN,"DVI page number output on\n",p);
 	    } else {
@@ -49,7 +78,7 @@ bool DecodeArgs(int argc, char ** argv)
 	    }
 	    break;
 	  } else if (strncmp(p,"epth",4)==0) { /* Depth reporting */ 
-	    if (p[8] != '0') {
+	    if (p[4] != '0') {
 	      flags |= REPORT_DEPTH;
 	      Message(PARSE_STDIN,"Depth reporting on\n",p);
 	    } else {
@@ -110,7 +139,7 @@ bool DecodeArgs(int argc, char ** argv)
 	if (strcmp(p,"elp") == 0 ) {
 	  break;
 	} else if (strncmp(p,"eight",5) == 0 ) { /* Height reporting */ 
-	  if (p[8] != '0') {
+	  if (p[5] != '0') {
 	    flags |= REPORT_HEIGHT;
 	    Message(PARSE_STDIN,"Height reporting on\n",p);
 	  } else {
@@ -129,7 +158,7 @@ bool DecodeArgs(int argc, char ** argv)
         break ;
       case 'e':
 	if (strncmp(p,"xpand-bbox",10) == 0 ) {	
-	  if (p[8] != '0') {
+	  if (p[10] != '0') {
 	    flags |= EXPAND_BBOX;
 	    Message(PARSE_STDIN,"BBox expansion on\n",p);
 	  } else {
@@ -160,7 +189,7 @@ bool DecodeArgs(int argc, char ** argv)
 #if HAVE_GDIMAGECREATETRUECOLOR
 	/* Truecolor */
 	if (strncmp(p,"ruecolor",8)==0) { 
-	  if (p[9] != '0') {
+	  if (p[8] != '0') {
 	    flags |= RENDER_TRUECOLOR; 
 	    Message(PARSE_STDIN,"Truecolor mode on\n",p);
 	  } else { 
@@ -199,7 +228,7 @@ bool DecodeArgs(int argc, char ** argv)
 	break;
       case 'c':
 	if (strncmp(p,"acheimages",10)==0) { 
-	  if (p[11] != '0') {
+	  if (p[10] != '0') {
 	    flags |= CACHE_IMAGES;
 	    Message(PARSE_STDIN,"Caching images\n",p);
 	  } else { 
@@ -220,10 +249,10 @@ bool DecodeArgs(int argc, char ** argv)
 	    background(p);
 	  if (borderwidth>=0) 
 	    Message(PARSE_STDIN,"Background: rgb %d,%d,%d\n",
-		    bRed,bGreen,bBlue);
+		    cstack[0].red,cstack[0].green,cstack[0].blue);
 	  else 
 	    Message(PARSE_STDIN,"Transp. background (fallback rgb %d,%d,%d)\n",
-		    bRed,bGreen,bBlue);
+		    cstack[0].red,cstack[0].green,cstack[0].blue);
 	  break;
 	} else if (strcmp(p, "dpi")==0) {
 	  p+=3;
@@ -249,7 +278,8 @@ bool DecodeArgs(int argc, char ** argv)
 	  if (*p == 0 && argv[i+1])
 	    p = argv[++i] ;
 	  resetcolorstack(p);
-	  Message(PARSE_STDIN,"Foreground: rgb %d,%d,%d\n",Red,Green,Blue);
+	  Message(PARSE_STDIN,"Foreground: rgb %d,%d,%d\n",
+		  cstack[1].red,cstack[1].green,cstack[1].blue);
 	  break;
 #ifdef HAVE_FT2
 	} else if ( strncmp(p,"reetype",7) == 0 ) { /* -freetype activation */
@@ -279,23 +309,58 @@ bool DecodeArgs(int argc, char ** argv)
 	Message(PARSE_STDIN,"Magstep: %d\n",usermag);
 	/*overridemag = (c == 'x' ? 1 : -1) ;*/
 	break ;
+      case 'g' :
+	if (strncmp(p,"amma",4)==0) { /* --gamma correction */ 
+	  double gamma=0.0;
+
+	  p+=4;
+	  if (*p == 0 && argv[i+1])
+	    p = argv[++i];
+	  if (p!=NULL)
+	    gamma=atof(p);
+	  if (gamma==0.0) {
+	    Warning("Bad gamma value, default is %f",DEFAULT_GAMMA);
+	    gamma=DEFAULT_GAMMA;
+	  }
+	  Gamma(gamma);
+	  Message(PARSE_STDIN,"Gamma value is %s\n", gamma);
+	  break;
+#ifdef HAVE_GDIMAGEGIF
+	} else if (strncmp(p,"if",2)==0) { /* --gif output */ 
+	  flags |= GIF_OUTPUT;
+	  Message(PARSE_STDIN,"GIF output\n");
+	  break;
+#endif
+	}
+	goto DEFAULT;
       case 'p' :
 	if (*p == 'p') {  /* a -pp specifier for a page list */
-	  ppused=_TRUE;
+	  ppused=true;
 	  p++ ;
 	  if (*p == 0 && argv[i+1])
 	    p = argv[++i];
 	  Message(PARSE_STDIN,"Page list: %s\n",p);
 	  if (ParsePages(p))
 	    Fatal("bad page list specifier (-pp).");
+	} else if (strncmp(p,"ng",2)==0) { /* --png output */ 
+	  flags &= ~GIF_OUTPUT;
+	  Message(PARSE_STDIN,"PNG output\n");
+	} else if (strncmp(p,"icky",4)==0) { 
+	  if (p[4] != '0') {
+	    flags |= MODE_PICKY;
+	    Message(PARSE_STDIN,"No images output for pages with warnings\n",p);
+	  } else {
+	    flags &= ~MODE_PICKY;
+	    Message(PARSE_STDIN,"Images output even for pages with warnings\n");
+	  }
 	} else {   /* a -p specifier for first page */
 	  int32_t firstpage;
-	  bool abspage=_FALSE;
+	  bool abspage=false;
 
 	  if (*p == 0 && argv[i+1])
 	    p = argv[++i] ;
 	  if (*p == '=') {
-	    abspage=_TRUE;
+	    abspage=true;
 	    p++ ;
 	  }
 	  firstpage = atoi(p);
@@ -303,15 +368,47 @@ bool DecodeArgs(int argc, char ** argv)
 	  Message(PARSE_STDIN,"First page: %d\n",firstpage);
 	}
 	break ;
+      case 's' :
+	if (strncmp(p,"trict",5)==0) { 
+	  if (p[5] != '0') {
+	    flags |= MODE_STRICT;
+	    Message(PARSE_STDIN,"Warnings are fatal\n",p);
+	  } else {
+	    flags &= ~MODE_STRICT;
+	    Message(PARSE_STDIN,"Warnings are not fatal\n");
+	  }
+	} else
+	  goto DEFAULT;
+	break ;
+      case 'n' :
+	if (strncmp(p,"oghostscript",12)==0) { 
+	  if (p[12] != '0') {
+	    flags |= NO_GHOSTSCRIPT;
+	    Message(PARSE_STDIN,"No GhostScript calls\n",p);
+	  } else {
+	    flags &= ~NO_GHOSTSCRIPT;
+	    Message(PARSE_STDIN,"GhostScript calls made\n");
+	  }
+	} else if (strncmp(p,"ogssafer",8)==0) { 
+	  if (p[8] != '0') {
+	    flags |= NO_GSSAFER;
+	    Message(PARSE_STDIN,"GhostScript calls does not use -dSAFER\n",p);
+	  } else {
+	    flags &= ~NO_GSSAFER;
+	    Message(PARSE_STDIN,"GhostScript calls use -dSAFER\n");
+	  }
+	} else
+	  goto DEFAULT;
+	break ;
       case 'l':
 	{
 	  int32_t lastpage;
-	  bool abspage=_FALSE;
+	  bool abspage=false;
 
 	  if (*p == 0 && argv[i+1])
 	    p = argv[++i] ;
 	  if (*p == '=') {
-	    abspage=_TRUE;
+	    abspage=true;
 	    p++ ;
 	  } 
 	  lastpage = atoi(p);
@@ -326,10 +423,13 @@ bool DecodeArgs(int argc, char ** argv)
 	  flags |= BE_NONQUIET;
 	break;
       case 'r':       /* process pages in reverse order */
-	if (*p != '0') 
+	if (*p != '0') {
 	  Message(PARSE_STDIN,"Reverse order\n");
-	else
+	  Reverse(true);
+	} else {
 	  Message(PARSE_STDIN,"Normal order\n");
+	  Reverse(false);
+	}
         break;
       case 'v':    /* verbatim mode */
 	if (strcmp(p, "ersion")==0) {
@@ -365,7 +465,7 @@ named COPYING and dvipng.c.");
 	if (*p == 0 && argv[i+1])
 	  p = argv[++i] ;
         shrinkfactor = atoi(p);
-	Message(PARSE_STDIN,"Shrinkfactor: %d\n",shrinkfactor);
+	Message(PARSE_STDIN,"Quality: %d\n",shrinkfactor);
 	break;
 #ifdef HAVE_GDIMAGEPNGEX
       case 'z':
@@ -385,10 +485,13 @@ named COPYING and dvipng.c.");
     }
   }
 
-  if (programname)
-    Message(BE_NONQUIET,"This is %s Copyright 2002-2004 Jan-ke Larsson\n",
-	    PACKAGE_STRING);
-
+  if (programname) {
+    Message(BE_NONQUIET,"This is %s",programname);
+    if (strcmp(basename(programname),PACKAGE_NAME)!=0)
+      Message(BE_NONQUIET," (%s)", PACKAGE_NAME);
+    Message(BE_NONQUIET," %s Copyright 2002-2004 Jan-ke Larsson\n",
+	    PACKAGE_VERSION);
+  }
   if (dviname != NULL) {
     if (dvi != NULL && dvi->filep != NULL) 
       DVIClose(dvi);
@@ -426,8 +529,14 @@ named COPYING and dvipng.c.");
     fprintf(stdout,"  --follow*    Follow mode\n");
 #ifdef HAVE_FT2
     fprintf(stdout,"  --freetype*  FreeType font rendering (default on)\n");
+#endif
+#ifdef HAVE_GDIMAGEGIF
+    fprintf(stdout,"  --gif        Output GIF images (dvigif default)\n");
 #endif
     fprintf(stdout,"  --height*    Output the image height on stdout\n");
+#ifdef HAVE_GDIMAGEGIF
+    fprintf(stdout,"  --png        Output PNG images (dvipng default)\n");
+#endif
 #ifdef HAVE_LIBT1
     fprintf(stdout,"  --t1lib*     T1lib font rendering (default on)\n");
 #endif
@@ -456,16 +565,6 @@ named COPYING and dvipng.c.");
     ParsePages("-");
   return((flags & PARSE_STDIN) != 0);
 }
-/*
-char * xmalloc(unsigned size)
-{
-  char *mem;
-  
-  if ((mem = malloc(size)) == NULL)
-    Fatal("cannot allocate %d bytes", size);
-  return mem;
-}
-*/
 
 void DecodeString(char *string)
 {
@@ -541,7 +640,7 @@ void Fatal (char *fmt, ...)
   if (libt1)
     (void) T1_CloseLib(); /* at this point, ignore error */
 #endif  
-  exit(EXIT_FAILURE);
+  exit(EXIT_FATAL);
 }
 
 /*-->Warning*/
@@ -578,3 +677,67 @@ void Message(int activeflags, char *fmt, ...)
   va_end(args);
 }
 
+
+bool MmapFile (char *filename,struct filemmap *fmmap)
+{
+#ifndef MIKTEX
+  struct stat stat;
+#endif
+
+  DEBUG_PRINT(DEBUG_DVI,("\n  OPEN FILE:\t'%s'", filename));
+  fmmap->mmap=NULL;
+#ifndef MIKTEX
+  if ((fmmap->fd = open(filename,O_RDONLY)) == -1) {
+    Warning("cannot open file <%s>", filename);
+    return(true);
+  }
+  fstat(fmmap->fd,&stat);
+  fmmap->size=stat.st_size;
+  fmmap->mmap = mmap(NULL,fmmap->size, PROT_READ, MAP_SHARED,fmmap->fd,0);
+  if (fmmap->mmap == (unsigned char *)-1) {
+    Warning("cannot mmap file <%s>",filename);
+    fmmap->mmap=NULL;
+    close(fmmap->fd);
+    return(true);
+  }
+#else /* MIKTEX */
+  fmmap->hFile = CreateFile(filename, GENERIC_READ, 0, 0, OPEN_EXISTING,
+			    FILE_FLAG_RANDOM_ACCESS, 0);
+  if (fmmap->hFile == INVALID_HANDLE_VALUE) {
+    Warning("cannot open file <%s>", filename);
+    return(true);
+  }
+  fmmap->size = GetFileSize(hFile, 0);
+  fmmap->hMap = CreateFileMapping(hFile, 0, PAGE_READONLY, 0, 0, 0);
+  if (fmmap->hMap == 0) {
+    CloseHandle (fmmap->hFile);
+    Warning("cannot CreateFileMapping() file <%s>", filename);
+    return(true);
+  }
+  fmmap->mmap = MapViewOfFile(hMap, FILE_MAP_READ, 0, 0, 0);
+  if (fmmap->mmap == NULL) {
+    Warning("cannot MapViewOfFile() file <%s>", filename);
+    CloseHandle (fmmap->hMap);
+    CloseHandle (fmmap->hFile);
+    return(true);
+  }
+#endif  /* MIKTEX */
+  return(false);
+}
+
+void UnMmapFile(struct filemmap* fmmap)
+{
+  if (fmmap->mmap!=NULL) {
+#ifndef MIKTEX
+    if (munmap(fmmap->mmap,fmmap->size))
+      Warning("cannot munmap file at 0x%X",fmmap->mmap);
+    if (close(fmmap->fd))
+      Warning("cannot close file descriptor %d",fmmap->fd);
+#else  /* MIKTEX */
+    UnmapViewOfFile (fmmap->mmap);
+    CloseHandle (fmmap->hMap);
+    CloseHandle (fmmap->hFile);
+#endif	/* MIKTEX */
+  }
+  fmmap->mmap=NULL;
+}
