@@ -1,5 +1,30 @@
+/* special.c */
+
+/************************************************************************
+
+  Part of the dvipng distribution
+
+  This program is free software; you can redistribute it and/or modify
+  it under the terms of the GNU General Public License as published by
+  the Free Software Foundation; either version 2 of the License, or
+  (at your option) any later version.
+
+  This program is distributed in the hope that it will be useful, but
+  WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+  General Public License for more details.
+
+  You should have received a copy of the GNU General Public License
+  along with this program; if not, write to the Free Software
+  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
+  02111-1307, USA.
+
+  Copyright  2002-2004 Jan-ke Larsson
+
+************************************************************************/
+
 #include "dvipng.h"
-#include <sys/wait.h>
+#include <fcntl.h>
 #if HAVE_ALLOCA_H
 # include <alloca.h>
 #endif
@@ -8,34 +33,59 @@ gdImagePtr
 ps2png(const char *psfile, int hresolution, int vresolution, 
        int urx, int ury, int llx, int lly)
 {
-  int status, downpipe[2], uppipe[2];
+#ifndef MIKTEX
+  int downpipe[2], uppipe[2];
   pid_t pid;
+#else
+  HANDLE hPngStream;
+  HANDLE hPsStream;
+  HANDLE hStdErr;
+  PROCESS_INFORMATION pi;
+  _TCHAR szCommandLine[2048];
+  _TCHAR szGsPath[_MAX_PATH];
+  int fd;
+#endif
+  FILE *psstream=NULL, *pngstream=NULL;
   char resolution[STRSIZE]; 
-  char devicesize[STRSIZE]; 
+  //  char devicesize[STRSIZE]; 
   /* For some reason, png256 gives inferior result */
   char *device="-sDEVICE=png16m";  
   gdImagePtr psimage=NULL;
   static bool showpage;
 
-  status=snprintf(resolution, STRSIZE, "-r%dx%d",hresolution,vresolution);
-  status=snprintf(devicesize, STRSIZE, "-g%dx%d",
-		  //(int)((sin(atan(1.0))+1)*
-		  (urx - llx)*hresolution/72,//), 
-		  //(int)((sin(atan(1.0))+1)*
-    (ury - lly)*vresolution/72);//);
+  sprintf(resolution, "-r%dx%d",hresolution,vresolution);
+  /* Future extension for \rotatebox
+  status=sprintf(devicesize, "-g%dx%d",
+		 //(int)((sin(atan(1.0))+1)*
+		 (urx - llx)*hresolution/72,//), 
+		 //(int)((sin(atan(1.0))+1)*
+		 (ury - lly)*vresolution/72);//);
+  */
   /* png16m being the default, this code is not needed
    * #ifdef HAVE_GDIMAGECREATETRUECOLOR
    * if (flags & RENDER_TRUECOLOR) 
    * device="-sDEVICE=png16m";
    * #endif  
    */
-  if (status>0 && status<STRSIZE 
-      && pipe(downpipe)==0 && pipe(uppipe)==0) { /* Ready to fork */
-    pid = fork ();
-    if (pid == 0) { /* Child process.  Execute gs. */       
-      close(downpipe[1]);
-      dup2(downpipe[0], STDIN_FILENO);
-      close(downpipe[0]);
+
+#ifndef MIKTEX
+  if (pipe(downpipe) || pipe(uppipe)) return(NULL);
+  /* Ready to fork */
+  pid = fork ();
+  if (pid == 0) { /* Child process.  Execute gs. */       
+    close(downpipe[1]);
+    dup2(downpipe[0], STDIN_FILENO);
+    close(downpipe[0]);
+#ifdef DEBUG
+    if (flags & NO_GSSAFER) {
+      DEBUG_PRINT(DEBUG_GS,
+		  ("\n  GS CALL:\t%s %s %s %s %s %s %s %s %s %s ",// %s",
+		   GS_PATH, device, resolution, //devicesize,
+		   "-dBATCH", "-dNOPAUSE", "-q", 
+		   "-sOutputFile=-", 
+		   "-dTextAlphaBits=4", "-dGraphicsAlphaBits=4",
+		   "-"));
+    } else {
       DEBUG_PRINT(DEBUG_GS,
 		  ("\n  GS CALL:\t%s %s %s %s %s %s %s %s %s %s %s ",// %s",
 		   GS_PATH, device, resolution, //devicesize,
@@ -43,67 +93,135 @@ ps2png(const char *psfile, int hresolution, int vresolution,
 		   "-sOutputFile=-", 
 		   "-dTextAlphaBits=4", "-dGraphicsAlphaBits=4",
 		   "-"));
-      close(uppipe[0]);
-      dup2(uppipe[1], STDOUT_FILENO);
-      close(uppipe[1]);
+    }
+#endif
+    close(uppipe[0]);
+    dup2(uppipe[1], STDOUT_FILENO);
+    close(uppipe[1]);
+    if (flags & NO_GSSAFER) 
+      execl (GS_PATH, GS_PATH, device, resolution, //devicesize,
+	     "-dBATCH", "-dNOPAUSE", "-q", 
+	     "-sOutputFile=-", 
+	     "-dTextAlphaBits=4", "-dGraphicsAlphaBits=4",
+	     "-",NULL);
+    else
       execl (GS_PATH, GS_PATH, device, resolution, //devicesize,
 	     "-dBATCH", "-dNOPAUSE", "-dSAFER", "-q", 
 	     "-sOutputFile=-", 
 	     "-dTextAlphaBits=4", "-dGraphicsAlphaBits=4",
 	     "-",NULL);
-      _exit (EXIT_FAILURE);
+    _exit (EXIT_FAILURE);
+  }
+  /* Parent process. */
+  
+  close(downpipe[0]);
+  psstream=fdopen(downpipe[1],"wb");
+  if (psstream == NULL) 
+    close(downpipe[1]);
+  close(uppipe[1]);
+  pngstream=fdopen(uppipe[0],"rb");
+  if (pngstream == NULL) 
+    close(uppipe[0]);
+#else /* MIKTEX */
+  if (! miktex_find_miktex_executable("mgs.exe", szGsPath)) {
+      Warning("Ghostscript could not be found");
+      return(NULL);
+  }
+  if (flags & NO_GSSAFER) {
+    DEBUG_PRINT(DEBUG_GS,
+		("\n  GS CALL:\t%s %s %s %s %s %s %s %s %s %s ",// %s",
+		 szGsPath, device, resolution, //devicesize,
+		 "-dBATCH", "-dNOPAUSE", "-q", 
+		 "-sOutputFile=-", 
+		 "-dTextAlphaBits=4", "-dGraphicsAlphaBits=4",
+		 "-"));
+    sprintf(szCommandLine,"\"%s\" %s %s %s %s %s %s %s %s %s",// %s",
+	    szGsPath, device, resolution, //devicesize,
+	    "-dBATCH", "-dNOPAUSE", "-q", 
+	    "-sOutputFile=-", 
+	    "-dTextAlphaBits=4", "-dGraphicsAlphaBits=4",
+	    "-");
+  } else {
+    DEBUG_PRINT(DEBUG_GS,
+		("\n  GS CALL:\t%s %s %s %s %s %s %s %s %s %s %s ",// %s",
+		 szGsPath, device, resolution, //devicesize,
+		 "-dBATCH", "-dNOPAUSE", "-dSAFER", "-q", 
+		 "-sOutputFile=-", 
+		 "-dTextAlphaBits=4", "-dGraphicsAlphaBits=4",
+		 "-"));
+    sprintf(szCommandLine,"\"%s\" %s %s %s %s %s %s %s %s %s %s",// %s",
+	    szGsPath, device, resolution, //devicesize,
+	    "-dBATCH", "-dNOPAUSE", "-dSAFER", "-q", 
+	    "-sOutputFile=-", 
+	    "-dTextAlphaBits=4", "-dGraphicsAlphaBits=4",
+	    "-");
+  }
+  if (! miktex_start_process_3(szCommandLine, &pi, INVALID_HANDLE_VALUE,
+			       &hPsStream, &hPngStream, &hStdErr, 0)) {
+      Warning("Ghostscript could not be started");
+      return(NULL);
+  }
+  CloseHandle (pi.hThread);
+  fd = _open_osfhandle((intptr_t)hPsStream, _O_WRONLY);
+  if (fd >= 0) { 
+    psstream = _tfdopen(fd, "wb");
+    if (psstream == NULL) 
+      _close (fd);
+  }
+  fd = _open_osfhandle((intptr_t)hPngStream, _O_RDONLY);
+  if (fd >= 0) {
+    pngstream = _tfdopen(fd, "rb");
+    if (pngstream == NULL) 
+      _close (fd);
+  }
+#endif 
+  if (psstream) {
+    DEBUG_PRINT(DEBUG_GS,("\n  PS CODE:\t<</PageSize[%d %d]/PageOffset[%d %d[1 1 dtransform exch]{0 ge{neg}if exch}forall]>>setpagedevice",
+			  urx - llx, ury - lly,llx,lly));
+    fprintf(psstream, "<</PageSize[%d %d]/PageOffset[%d %d[1 1 dtransform exch]{0 ge{neg}if exch}forall]>>setpagedevice\n",
+	    urx - llx, ury - lly,llx,lly);
+    if ( cstack[0].red < 255 || cstack[0].green < 255 || cstack[0].blue < 255 ) {
+      DEBUG_PRINT(DEBUG_GS,("\n  PS CODE:\tgsave %f %f %f setrgbcolor clippath fill grestore",
+			    cstack[0].red/255.0, cstack[0].green/255.0, cstack[0].blue/255.0));
+      fprintf(psstream, "gsave %f %f %f setrgbcolor clippath fill grestore",
+	      cstack[0].red/255.0, cstack[0].green/255.0, cstack[0].blue/255.0);
     }
-    else if (pid > 0) { /* Parent process. */
-      FILE *psstream, *pngstream;
-      
-      close(downpipe[0]);
-      psstream=fdopen(downpipe[1],"w");
-      if (psstream) {
-	DEBUG_PRINT(DEBUG_GS,("\n  PS CODE:\t<</PageSize[%d %d]/PageOffset[%d %d[1 1 dtransform exch]{0 ge{neg}if exch}forall]>>setpagedevice",
-	       urx - llx, ury - lly,llx,lly));
-       fprintf(psstream, "<</PageSize[%d %d]/PageOffset[%d %d[1 1 dtransform exch]{0 ge{neg}if exch}forall]>>setpagedevice\n",
-               urx - llx, ury - lly,llx,lly);
-	if ( bRed < 255 || bGreen < 255 || bBlue < 255 ) {
-	  DEBUG_PRINT(DEBUG_GS,("\n  PS CODE:\tgsave %f %f %f setrgbcolor clippath fill grestore",
-		       bRed/256.0, bGreen/256.0, bBlue/256.0));
-	  fprintf(psstream, "gsave %f %f %f setrgbcolor clippath fill grestore",
-		  bRed/256.0, bGreen/256.0, bBlue/256.0);
-	}
-	DEBUG_PRINT(DEBUG_GS,("\n  PS CODE:\t(%s) run", psfile));
-	fprintf(psstream, "(%s) run\n", psfile);
-	if (showpage) {
-	  DEBUG_PRINT(DEBUG_GS,("\n  PS CODE:\tshowpage"));
-	  fprintf(psstream, "showpage\n");
-	}
-	DEBUG_PRINT(DEBUG_GS,("\n  PS CODE:\tquit"));
-	fprintf(psstream, "quit\n");
-	fclose(psstream);
-      }
-      close(uppipe[1]);
-      pngstream=fdopen(uppipe[0],"r");
-      if (pngstream) {
-	psimage = gdImageCreateFromPng(pngstream);
-	fclose(pngstream);
-      }
+    DEBUG_PRINT(DEBUG_GS,("\n  PS CODE:\t(%s) run", psfile));
+    fprintf(psstream, "(%s) run\n", psfile);
+    if (showpage) {
+      DEBUG_PRINT(DEBUG_GS,("\n  PS CODE:\tshowpage"));
+      fprintf(psstream, "showpage\n");
+    }
+    DEBUG_PRINT(DEBUG_GS,("\n  PS CODE:\tquit"));
+    fprintf(psstream, "quit\n");
+    fclose(psstream);
+  }
+  if (pngstream) {
+    psimage = gdImageCreateFromPng(pngstream);
+    fclose(pngstream);
+  }
+#ifdef MIKTEX
+  CloseHandle(pi.hProcess);
+#endif
 #ifdef HAVE_GDIMAGETRUECOLORTOPALETTE
 #ifdef HAVE_GDIMAGECREATETRUECOLOR
-      if (!flags & RENDER_TRUECOLOR)
+  if (!flags & RENDER_TRUECOLOR)
 #endif
-	gdImageTrueColorToPalette(psimage,0,256);
+    gdImageTrueColorToPalette(psimage,0,256);
 #endif
-      if (psimage == NULL) {
-	DEBUG_PRINT(DEBUG_GS,("\n  GS OUTPUT:\tNO IMAGE "));
-	if (!showpage) {
-	  showpage=TRUE;
-	  DEBUG_PRINT(DEBUG_GS,("(will try adding \"showpage\") "));
-	  psimage=ps2png(psfile, hresolution, vresolution, urx, ury, llx, lly);
-	  showpage=FALSE;
-	}
-      } else {
-	DEBUG_PRINT(DEBUG_GS,("\n  GS OUTPUT:\t%dx%d image ",
-			      gdImageSX(psimage),gdImageSY(psimage)));
-      }
+  if (psimage == NULL) {
+    DEBUG_PRINT(DEBUG_GS,("\n  GS OUTPUT:\tNO IMAGE "));
+    if (!showpage) {
+      showpage=true;
+      DEBUG_PRINT(DEBUG_GS,("(will try adding \"showpage\") "));
+      psimage=ps2png(psfile, hresolution, vresolution, urx, ury, llx, lly);
+      showpage=false;
     }
+#ifdef DEBUG
+  } else {
+    DEBUG_PRINT(DEBUG_GS,("\n  GS OUTPUT:\t%dx%d image ",
+			  gdImageSX(psimage),gdImageSY(psimage)));
+#endif
   }
   return psimage;
 }
@@ -182,42 +300,48 @@ void SetSpecial(char * special, int32_t length, int32_t hh, int32_t vv)
     if (hresolution==0) hresolution = vresolution = dpi;
     
     if (page_imagep != NULL) { /* Draw into image */
-      char* psfile = kpse_find_file(psname,kpse_pict_format,0);
-      char* pngname = NULL;
+      char* psfile;
       gdImagePtr psimage=NULL;
 
-      if (flags & CACHE_IMAGES) {
-	char* pngfile;
+      /*---------- Cache ----------*/
+      char* cachename = NULL;
+      gdImagePtr cacheimage=NULL;
 
-	pngname = malloc(sizeof(char)*(strlen(psname+5)));
-	if (pngname==NULL) 
+      TEMPSTR(psfile,kpse_find_file(psname,kpse_pict_format,0));
+      if (flags & CACHE_IMAGES) { /* Find cached image, if it exists */
+	char *cachefile,*separator;
+
+	cachename = alloca(sizeof(char)*(strlen(psname+5)));
+	if (cachename==NULL) 
 	  Fatal("Cannot allocate space for cached image filename");
-	strcpy(pngname,psname);
-	pngfile = strrchr(pngname,'.');
-	if (pngfile==NULL)
-	  strcat(pngname,".png");
-	else {
-	  pngfile[1] = 'p';
-	  pngfile[2] = 'n';
-	  pngfile[3] = 'g';
-	  pngfile[4] = '\0';
-	}
-	pngfile = kpse_find_file(pngname,kpse_pict_format,0);
-	if (pngfile!=NULL) {
-	  FILE* pngfilep = fopen(pngfile,"rb");
+	strcpy(cachename,psname);
+	separator = strrchr(cachename,'.');
+	if (separator!=NULL)
+	  *separator='\0';
+	strcat(cachename,".png");
+	cachefile = kpse_find_file(cachename,kpse_pict_format,0);
+	if (cachefile!=NULL) {
+	  FILE* cachefilep = fopen(cachefile,"rb");
 
-	  if (pngfilep!=NULL) {
-	    psimage = gdImageCreateFromPng(pngfilep);
-	    fclose(pngfilep);
+	  if (cachefilep!=NULL) {
+	    cacheimage = gdImageCreateFromPng(cachefilep);
+	    fclose(cachefilep);
 	  }
-	  free(pngfile);
+	  free(cachefile);
 	}
+	psimage = cacheimage;
       }
+      /*---------- End Cache ----------*/
       Message(BE_NONQUIET,"<%s",psname);
       if (psimage==NULL) {
+	/* No cached image, convert postscript */
 	if (psfile == NULL) {
 	  Warning("PS file %s not found, image will be left blank", psname );
 	  flags |= PAGE_GAVE_WARN;
+	} else if (flags & NO_GHOSTSCRIPT) {
+	    Warning("GhostScript calls disallowed by --noghostscript", 
+		    psfile );
+	    flags |= PAGE_GAVE_WARN;
 	} else {
 	  psimage = ps2png(psfile, hresolution, vresolution, 
 			   urx, ury, llx, lly);
@@ -228,14 +352,18 @@ void SetSpecial(char * special, int32_t length, int32_t hh, int32_t vv)
 	  }
 	}
       }
-      if (pngname !=NULL && psimage != NULL) {
-	FILE* pngfilep = fopen(pngname,"wb");
-	if (pngfilep!=NULL) {
-	  gdImagePng(psimage,pngfilep);
-	  fclose(pngfilep);
+      /*---------- Store Cache ----------*/
+      if (flags & CACHE_IMAGES && cachename !=NULL && 
+	  cacheimage==NULL && psimage != NULL) {
+	/* Cache image not found, save converted postscript */
+	FILE* cachefilep = fopen(cachename,"wb");
+	if (cachefilep!=NULL) {
+	  gdImagePng(psimage,cachefilep);
+	  fclose(cachefilep);
 	} else
 	  Warning("Unable to cache %s as PNG", psfile );
       } 
+      /*---------- End Store Cache ----------*/
       if (psimage!=NULL) {
 	DEBUG_PRINT(DEBUG_DVI,
 		    ("\n  PS-PNG INCLUDE \t%s (%d,%d) res %dx%d at (%d,%d)",
@@ -250,7 +378,7 @@ void SetSpecial(char * special, int32_t length, int32_t hh, int32_t vv)
 	gdImageDestroy(psimage);
       }
       Message(BE_NONQUIET,">");
-    } else { /* Not PASS_DRAW */
+    } else { /* Don't draw */
       int pngheight,pngwidth;
       
       /* Convert from postscript 72 dpi resolution to our given resolution */
@@ -321,11 +449,9 @@ void SetSpecial(char * special, int32_t length, int32_t hh, int32_t vv)
 	      hh, vv, length,special);
     return;
   }
-  if ( page_imagep != NULL || flags & NO_IMAGE_ON_WARN ) {
+  if ( page_imagep != NULL || flags & MODE_PICKY ) {
     Warning("at (%ld,%ld) unimplemented \\special{%.*s}.",
 	    hh, vv, length,special);
     flags |= PAGE_GAVE_WARN;
   }
 }
-
-
