@@ -1,8 +1,29 @@
-#include "dvipng.h"
-#include <fcntl.h> // open/close
-#include <sys/mman.h>
-#include <sys/stat.h>
+/* vf.c */
 
+/************************************************************************
+
+  Part of the dvipng distribution
+
+  This program is free software; you can redistribute it and/or modify
+  it under the terms of the GNU General Public License as published by
+  the Free Software Foundation; either version 2 of the License, or
+  (at your option) any later version.
+
+  This program is distributed in the hope that it will be useful, but
+  WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+  General Public License for more details.
+
+  You should have received a copy of the GNU General Public License
+  along with this program; if not, write to the Free Software
+  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
+  02111-1307, USA.
+
+  Copyright  2002-2004 Jan-ke Larsson
+
+************************************************************************/
+
+#include "dvipng.h"
 
 #define VF_ID 202
 #define LONG_CHAR 242
@@ -11,11 +32,11 @@ int32_t SetVF(int32_t c)
 {
   struct font_entry* currentvf;
   unsigned char *command,*end;
-  struct vf_char* ptr=currentfont->chr[c];
+  struct char_entry* ptr=currentfont->chr[c];
 
   currentvf=currentfont;
   BeginVFMacro(currentvf);
-  command = ptr->mmap;
+  command = ptr->data;
   end = command + ptr->length;
   while (command < end)  {
     DEBUG_PRINT(DEBUG_DVI,("\n  VF MACRO:\t%s ", dvi_commands[*command]));
@@ -31,31 +52,25 @@ int32_t SetVF(int32_t c)
 
 void InitVF(struct font_entry * tfontp)
 {
-  struct stat stat;
   unsigned char* position;
   int length;
-  struct vf_char *tcharptr;  
+  struct char_entry *tcharptr;  
   uint32_t c=0;
   struct font_num *tfontnump;  /* temporary font_num pointer   */
   
   DEBUG_PRINT((DEBUG_DVI|DEBUG_VF),("\n  OPEN FONT:\t'%s'", tfontp->name));
   Message(BE_VERBOSE,"<%s>", tfontp->name);
-  if ((tfontp->filedes = open(tfontp->name,O_RDONLY)) == -1) 
-    Warning("font file %s could not be opened", tfontp->name);
-  fstat(tfontp->filedes,&stat);
-  tfontp->mmap = mmap(NULL,stat.st_size,
-      PROT_READ, MAP_SHARED,tfontp->filedes,0);
-  if (tfontp->mmap == (unsigned char *)-1) 
-    Fatal("cannot mmap VF file <%s> !\n",currentfont->name);
-  tfontp->end=tfontp->mmap+stat.st_size;
-  if (*(tfontp->mmap) != PRE) 
+  if (MmapFile(tfontp->name,&(tfontp->fmmap)))
+    Fatal("font file %s unusable", tfontp->name);
+  position=tfontp->fmmap.mmap;
+  if (*(position) != PRE) 
     Fatal("unknown font format in file <%s> !\n",currentfont->name);
-  if (*(tfontp->mmap+1) != VF_ID) 
+  if (*(position+1) != VF_ID) 
       Fatal( "wrong version of vf file!  (%d should be 202)\n",
-	     (int)*(tfontp->mmap+1));
+	     (int)*(position+1));
   DEBUG_PRINT(DEBUG_VF,("\n  VF_PRE:\t'%.*s'", 
-		(int)*(tfontp->mmap+2), tfontp->mmap+3));
-  position = tfontp->mmap+3 + *(tfontp->mmap+2);
+		(int)*(position+2), position+3));
+  position = position+3 + *(position+2);
   c=UNumRead(position, 4);
   DEBUG_PRINT(DEBUG_VF,(" %d", c));
   CheckChecksum (tfontp->c, c, tfontp->name);
@@ -67,7 +82,7 @@ void InitVF(struct font_entry * tfontp)
   position += 8;
   while(*position >= FNT_DEF1 && *position <= FNT_DEF4) {
     DEBUG_PRINT(DEBUG_VF,("\n  @%ld VF:\t%s", 
-		  (long)(position - tfontp->mmap), 
+		  (long)((void*)position - tfontp->fmmap.mmap), 
 		  dvi_commands[*position]));
     FontDef(position,tfontp);	
     length = dvi_commandlength[*position];
@@ -83,8 +98,8 @@ void InitVF(struct font_entry * tfontp)
   /* Read char definitions */
   while(*position < FNT_DEF1) {
     DEBUG_PRINT(DEBUG_VF,("\n@%ld VF CHAR:\t", 
-		 (long)(position - tfontp->mmap)));
-    tcharptr=xmalloc(sizeof(struct vf_char));
+		 (long)((void*)position - tfontp->fmmap.mmap)));
+    tcharptr=xmalloc(sizeof(struct char_entry));
     switch (*position) {
     case LONG_CHAR:
       tcharptr->length = UNumRead(position+1,4);
@@ -105,7 +120,7 @@ void InitVF(struct font_entry * tfontp)
     if (c > NFNTCHARS) /* Only positive for now */
       Fatal("vf character exceeds numbering limit");
     tfontp->chr[c] = tcharptr;
-    tcharptr->mmap=position;
+    tcharptr->data=position;
     position += tcharptr->length;
   }
 }
@@ -113,15 +128,10 @@ void InitVF(struct font_entry * tfontp)
 
 void DoneVF(struct font_entry *tfontp)
 {
-  int c=0;
+  int c=FIRSTFNTCHAR;
 
-  if (munmap(tfontp->mmap,tfontp->end-tfontp->mmap))
-    Warning("font file %s could not be munmapped", tfontp->name);
-  if (close(tfontp->filedes)) 
-    Warning("font file %s could not be closed", tfontp->name);
-  tfontp->mmap=NULL;
-  tfontp->filedes=-1;
-  while(c<NFNTCHARS-1) {
+  UnMmapFile(&(tfontp->fmmap));
+  while(c<=LASTFNTCHAR) {
     if (tfontp->chr[c]!=NULL) {
       free(tfontp->chr[c]);
       tfontp->chr[c]=NULL;
